/******************************************************************************
**
** FILE NAME    : ar9_ref_board.c
** PROJECT      : IFX UEIP
** MODULES      : BSP Basic
**
** DATE         : 27 May 2009
** AUTHOR       : Xu Liang
** DESCRIPTION  : source file for AR9
** COPYRIGHT    :       Copyright (c) 2009
**                      Infineon Technologies AG
**                      Am Campeon 1-12, 85579 Neubiberg, Germany
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date        $Author         $Comment
** 27 May 2009   Xu Liang        The first UEIP release
*******************************************************************************/



#include <linux/module.h>
#include <linux/types.h>
#include <linux/kallsyms.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <asm/prom.h>

#include <ifx_regs.h>
#include <ifx_types.h>
#include <ifx_board.h>
#include <irq.h>
#include <ifx_gpio.h>
#include <ifx_ledc.h>
#include <ifx_led.h>
#include <ifx_si.h>


struct ifx_si_eiu_config g_si_eiu_config = {
    .irq            = -1,   //  no serial input
    .intsync        = 0,
    .sampling_clk   = 0,
    .shift_clk      = 0,
    .group          = 0,
    .active_high    = 0,
};


#define MAX_DEFINED_GPIOS   40   /*--- XXX bei Bedarf erweitern ---*/
#define FIRST_HWREV         161  /*--- Erstes AR9-Produkt ---*/
#define LAST_HWREV          200  /*--- XXX bei Bedarf erweitern ---*/
#define HWREV_COUNT         (LAST_HWREV - FIRST_HWREV + 2) 
#define HWREV_INDEX(HWREV)  (HWREV - FIRST_HWREV + 1)

/*------------------------------------------------------------------------------------------*\
 * GPIO PIN to Module Mapping and default PIN configuration for each Hardware
 * module_id of last item in each config must be IFX_GPIO_PIN_AVAILABLE
\*------------------------------------------------------------------------------------------*/
struct ifx_gpio_ioctl_pin_config hw_gpio_configs[HWREV_COUNT][MAX_DEFINED_GPIOS] = {

  /*------------------------------------------------------------------------------------------*\
   * DEFAULT GPIO CONFIG
  \*------------------------------------------------------------------------------------------*/
    [0] = {
            /*------------------------------------------------------------------------------------------*\
              LEDs (alle aktive low)
            \*------------------------------------------------------------------------------------------*/
        {
            .name      = "gpio_led_power",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 12),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_internet",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 15),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_dect",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_wlan",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 5),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_info",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 3),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_info_red",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 13),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },

        {
            .name      = "gpio_button_dect",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(0, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN  | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET
        },
        {
            .name      = "gpio_button_wlan",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(0, 1),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN  | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET   | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR 
        },

        {
            .name      = "gpio_dect_page_button",
            .module_id = IFX_GPIO_MODULE_PAGE,
            .pin       = IFX_GPIO_PIN_ID(2,6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR
        },

            /*--------------------------------------------------------------------------------*\
              AVM DECT RESET  
            \*--------------------------------------------------------------------------------*/
        {
            .name      = "gpio_avm_dect_reset",
            .module_id = IFX_GPIO_MODULE_PIGLET,
            .pin       = IFX_GPIO_PIN_ID(3, 6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
            /*--------------------------------------------------------------------------------*\
              AVM DECT UART  
            \*--------------------------------------------------------------------------------*/
        {
            .name      = "gpio_uart_rx",
            .module_id = IFX_GPIO_MODULE_PIGLET,
            .pin       = IFX_GPIO_PIN_ID(0, 11),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR
        },
        {
            .name      = "gpio_uart_tx",
            .module_id = IFX_GPIO_MODULE_PIGLET,
            .pin       = IFX_GPIO_PIN_ID(0, 12),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },


            /*--------------------------------------------------------------------------------*\
              AVM PCMBUS
            \*--------------------------------------------------------------------------------*/
            
        {
            .name      = "gpio_tdm_fsc", // used as INPUT by default (FSC Slave)
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(0, 0),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET
        },
        {
            .name      = "gpio_tdm_do",
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(1, 9),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_tdm_di",
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(2, 9),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET 
        },
        {
            .name      = "gpio_tdm_dcl", // used as INPUT by default (DCL Slave)
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(2, 8),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET 
        },
        {
            .name      = "gpio_spi_cs2",
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(1, 6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR 
        },

        {
            .name      = "gpio_spi_cs4",
            .module_id = IFX_GPIO_MODULE_SPI_FLASH,
            .pin       = IFX_GPIO_PIN_ID(0, 10),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_spi_din",
            .module_id = IFX_GPIO_MODULE_SSC,
            .pin       = IFX_GPIO_PIN_ID(1, 0),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR
        },
        {
            .name      = "gpio_spi_dout",
            .module_id = IFX_GPIO_MODULE_SSC,
            .pin       = IFX_GPIO_PIN_ID(1, 1),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_spi_clk",
            .module_id = IFX_GPIO_MODULE_SSC,
            .pin       = IFX_GPIO_PIN_ID(1, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },

        {
            .name      = "gpio_tapi_fxo_rstq",
            .module_id = IFX_GPIO_MODULE_TAPI_FXO,
            .pin       = IFX_GPIO_PIN_ID(0, 12),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET
        },
        {
            .name      = "gpio_tapi_fxo_csq",
            .module_id = IFX_GPIO_MODULE_TAPI_FXO,
            .pin       = IFX_GPIO_PIN_ID(3, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },

            /*------------------------------------------------------------------------------------------*\
              USB:
            \*------------------------------------------------------------------------------------------*/
        {
            .name      = "gpio_usb_pwr_en_0",
            .module_id = IFX_GPIO_MODULE_USB,
            .pin       = IFX_GPIO_USB_VBUS1,
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_usb_pwr_en_1",
            .module_id = IFX_GPIO_MODULE_USB,
            .pin       = IFX_GPIO_USB_VBUS2,
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },

            /*------------------------------------------------------------------------------------------*\
              Ethernet + WLAN:
            \*------------------------------------------------------------------------------------------*/
        {
            .name      = "gpio_extphy_25mhz_clk",
            .module_id = IFX_GPIO_MODULE_EXTPHY_25MHZ_CLOCK,
            .pin       = IFX_GPIO_PIN_ID(0, 3),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_int",
            .module_id = IFX_GPIO_MODULE_EXTPHY_INT, 
            .pin       = IFX_GPIO_PIN_ID(2, 7), 
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET
        },
        {
            .name      = "gpio_extphy_mdio",
            .module_id = IFX_GPIO_MODULE_EXTPHY_MDIO,
            .pin       = IFX_GPIO_PIN_ID(2, 10),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN|IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR|IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_mdc",
            .module_id = IFX_GPIO_MODULE_EXTPHY_MDIO,
            .pin       = IFX_GPIO_PIN_ID(2, 11),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR|IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_reset_0",
            .module_id = IFX_GPIO_MODULE_EXTPHY_RESET,
            .pin       = IFX_GPIO_PIN_ID(2, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR| IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_reset_1",
            .module_id = IFX_GPIO_MODULE_EXTPHY_RESET,
            .pin       = IFX_GPIO_PIN_ID(2, 4),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR| IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },

        //  module_id of last item must be IFX_GPIO_PIN_AVAILABLE
        { .module_id = IFX_GPIO_PIN_AVAILABLE }
    }, /*--- DEFAULT GPIO CONFIG ---*/


  /*------------------------------------------------------------------------------------------*\
  \*------------------------------------------------------------------------------------------*/
    [1 ... (HWREV_COUNT-1)] = {
        { .module_id = IFX_GPIO_PIN_AVAILABLE }
    },
    

  /*------------------------------------------------------------------------------------------*\
   * GPIO CONFIG 7312
   * => Change in gpio_extphy_25mhz_clk
  \*------------------------------------------------------------------------------------------*/
    [HWREV_INDEX(189)] = {

            /*------------------------------------------------------------------------------------------*\
              LEDs (alle aktive low)
            \*------------------------------------------------------------------------------------------*/
        {
            .name      = "gpio_led_power",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 12),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_internet",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 15),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_dect",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_wlan",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 5),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_info",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 3),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },
        {
            .name      = "gpio_led_info_red",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(2, 13),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },

        {
            .name      = "gpio_button_wlan",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(0, 1),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN  | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET   | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR 
        },
        {
            .name      = "gpio_button_dect",
            .module_id = IFX_GPIO_MODULE_LED,
            .pin       = IFX_GPIO_PIN_ID(0, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN  | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET
        },

        {
            .name      = "gpio_dect_page_button",
            .module_id = IFX_GPIO_MODULE_PAGE,
            .pin       = IFX_GPIO_PIN_ID(2,6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR
        },

            /*--------------------------------------------------------------------------------*\
              AVM DECT RESET  
            \*--------------------------------------------------------------------------------*/
        {
            .name      = "gpio_avm_dect_reset",
            .module_id = IFX_GPIO_MODULE_PIGLET,
            .pin       = IFX_GPIO_PIN_ID(3, 6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
            /*--------------------------------------------------------------------------------*\
              AVM DECT UART  
            \*--------------------------------------------------------------------------------*/
        {
            .name      = "gpio_uart_rx",
            .module_id = IFX_GPIO_MODULE_PIGLET,
            .pin       = IFX_GPIO_PIN_ID(0, 11),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR
        },
        {
            .name      = "gpio_uart_tx",
            .module_id = IFX_GPIO_MODULE_PIGLET,
            .pin       = IFX_GPIO_PIN_ID(0, 12),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },


            /*--------------------------------------------------------------------------------*\
              AVM PCMBUS
            \*--------------------------------------------------------------------------------*/
            
        {
            .name      = "gpio_tdm_fsc", // used as INPUT by default (FSC Slave)
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(0, 0),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET
        },
        {
            .name      = "gpio_tdm_do",
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(1, 9),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_tdm_di",
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(2, 9),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET 
        },
        {
            .name      = "gpio_tdm_dcl", // used as INPUT by default (DCL Slave)
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(2, 8),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET 
        },
        {
            .name      = "gpio_spi_cs2",
            .module_id = IFX_GPIO_MODULE_PCMLINK,
            .pin       = IFX_GPIO_PIN_ID(1, 6),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR 
        },

        {
            .name      = "gpio_spi_cs4",
            .module_id = IFX_GPIO_MODULE_SPI_FLASH,
            .pin       = IFX_GPIO_PIN_ID(0, 10),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_spi_din",
            .module_id = IFX_GPIO_MODULE_SSC,
            .pin       = IFX_GPIO_PIN_ID(1, 0),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR
        },
        {
            .name      = "gpio_spi_dout",
            .module_id = IFX_GPIO_MODULE_SSC,
            .pin       = IFX_GPIO_PIN_ID(1, 1),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_spi_clk",
            .module_id = IFX_GPIO_MODULE_SSC,
            .pin       = IFX_GPIO_PIN_ID(1, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },

        {
            .name      = "gpio_tapi_fxo_rstq",
            .module_id = IFX_GPIO_MODULE_TAPI_FXO,
            .pin       = IFX_GPIO_PIN_ID(0, 12),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET
        },
        {
            .name      = "gpio_tapi_fxo_csq",
            .module_id = IFX_GPIO_MODULE_TAPI_FXO,
            .pin       = IFX_GPIO_PIN_ID(3, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDEN_SET | IFX_GPIO_IOCTL_PIN_CONFIG_PUDSEL_SET | IFX_GPIO_IOCTL_PIN_CONFIG_OUTPUT_SET
        },

            /*------------------------------------------------------------------------------------------*\
              USB:
            \*------------------------------------------------------------------------------------------*/
        {
            .name      = "gpio_usb_pwr_en_0",
            .module_id = IFX_GPIO_MODULE_USB,
            .pin       = IFX_GPIO_USB_VBUS1,
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_usb_pwr_en_1",
            .module_id = IFX_GPIO_MODULE_USB,
            .pin       = IFX_GPIO_USB_VBUS2,
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },

            /*------------------------------------------------------------------------------------------*\
              Ethernet + WLAN:
            \*------------------------------------------------------------------------------------------*/
        {
            .name      = "gpio_extphy_25mhz_clk",
            .module_id = IFX_GPIO_MODULE_EXTPHY_25MHZ_CLOCK,
            .pin       = IFX_GPIO_PIN_ID(0, 3),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | 
                         IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET | 
                         IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR | 
                         IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_int",
            .module_id = IFX_GPIO_MODULE_EXTPHY_INT, 
            .pin       = IFX_GPIO_PIN_ID(2, 7), 
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_SET
        },
        {
            .name      = "gpio_extphy_mdio",
            .module_id = IFX_GPIO_MODULE_EXTPHY_MDIO,
            .pin       = IFX_GPIO_PIN_ID(2, 10),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_IN|IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR|IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_mdc",
            .module_id = IFX_GPIO_MODULE_EXTPHY_MDIO,
            .pin       = IFX_GPIO_PIN_ID(2, 11),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_SET|IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR|IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_reset_0",
            .module_id = IFX_GPIO_MODULE_EXTPHY_RESET,
            .pin       = IFX_GPIO_PIN_ID(2, 2),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR| IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },
        {
            .name      = "gpio_extphy_reset_1",
            .module_id = IFX_GPIO_MODULE_EXTPHY_RESET,
            .pin       = IFX_GPIO_PIN_ID(2, 4),
            .config    = IFX_GPIO_IOCTL_PIN_CONFIG_DIR_OUT | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL0_CLEAR | IFX_GPIO_IOCTL_PIN_CONFIG_ALTSEL1_CLEAR| IFX_GPIO_IOCTL_PIN_CONFIG_OD_SET
        },

        //  module_id of last item must be IFX_GPIO_PIN_AVAILABLE
        { .module_id = IFX_GPIO_PIN_AVAILABLE }

    } /*--- GPIO CONFIG 7312 ---*/

};


struct ifx_gpio_ioctl_pin_config *g_board_gpio_pin_map = hw_gpio_configs[0];
EXPORT_SYMBOL(g_board_gpio_pin_map);

void set_gpio_config(void) 
{
    unsigned int hwrev;
    char *s, *p;
    struct ifx_gpio_ioctl_pin_config *m = g_board_gpio_pin_map;

    s = prom_getenv("HWRevision");
    if (s) {
		hwrev = simple_strtoul(s, &p, 10);
	} else {
	  printk("[%s] error: no HWRevision detected in environment variables\n",__FUNCTION__);
	  BUG_ON(1);
	}

    if((hwrev <= LAST_HWREV) && hw_gpio_configs[HWREV_INDEX(hwrev)] && (hw_gpio_configs[HWREV_INDEX(hwrev)][0].module_id != IFX_GPIO_PIN_AVAILABLE)) {
        g_board_gpio_pin_map = hw_gpio_configs[HWREV_INDEX(hwrev)];
	  printk("[%s] Using specific gpio table for HW-Rev %d\n",__FUNCTION__, hwrev);
    }
    else {
	  printk("[%s] Using default gpio table for HW-Rev %d\n",__FUNCTION__, hwrev);
    }
}



struct ifx_ledc_config_param g_board_ledc_hw_config = {
    .operation_mask         = IFX_LEDC_CFG_OP_UPDATE_SOURCE | IFX_LEDC_CFG_OP_BLINK | IFX_LEDC_CFG_OP_UPDATE_CLOCK | IFX_LEDC_CFG_OP_STORE_MODE | IFX_LEDC_CFG_OP_SHIFT_CLOCK | IFX_LEDC_CFG_OP_DATA_OFFSET | IFX_LEDC_CFG_OP_NUMBER_OF_LED | IFX_LEDC_CFG_OP_DATA | IFX_LEDC_CFG_OP_MIPS0_ACCESS | IFX_LEDC_CFG_OP_DATA_CLOCK_EDGE,
    .source_mask            = 3,
    .source                 = 0,    //  by default all LEDs controlled by LEDC DATA
    .blink_mask             = (1 << 16) - 1,
    .blink                  = 0,    //  disable blink for all LEDs
    .update_clock           = LED_CON1_UPDATE_SRC_SOFTWARE,
    .fpid                   = 0,
    .store_mode             = 0,    //  single store
    .fpis                   = 0,
    .data_offset            = 0,
    .number_of_enabled_led  = 16,
    .data_mask              = (1 << 16) - 1,
    .data                   = (1 << 0) | (1 << 1) | (1 << 14),  // LED0,1 - 1 is turning off, LED14 - Warning LED turns on by default
    .mips0_access_mask      = (1 << 16) - 1,
    .mips0_access           = (1 << 16) - 1,
    .f_data_clock_on_rising = 0,    //  falling edge
};
EXPORT_SYMBOL(g_board_ledc_hw_config);

struct ifx_led_device g_board_led_hw_config[] = {
    {
        .default_trigger    = NULL,
        .phys_id            = 0,
        .value_on           = 0,
        .value_off          = 1,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 1,
        .value_on           = 0,
        .value_off          = 1,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 2,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 3,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 4,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 5,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 6,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 7,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 8,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 9,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 10,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 11,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 12,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 13,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 14,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .default_trigger    = NULL,
        .phys_id            = 15,
        .value_on           = 1,
        .value_off          = 0,
        .flags              = IFX_LED_DEVICE_FLAG_PHYS_LEDC,
    },
    {
        .flags              = IFX_LED_DEVICE_FLAG_INVALID,
    }
};
EXPORT_SYMBOL(g_board_led_hw_config);





